
/*
 *  split.c
 *  Copyright (C) 1999 Brad Proctor
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: split.c,v 1.5 1999/06/21 15:54:59 brad Exp $
 */

#include <stdio.h>
#include <string.h>
#include "version.h"

#define BUF_SIZE  65536

static char help_text[] =
{
   "Usage: split [-l line_count][-a suffix_length][file[name]]\n"
   "       split -b n[k|m][-a suffix_length][file[name]]\n"
   "       split [-line_count][-a suffix_length][file[name]]\n\n"
   "  -a suffix_length  Use suffix_length letters for the split filenames\n"
   "  -b n              Split a file into n bytes in size\n"
   "  -b nk             Split a file into n*1024 bytes in size\n"
   "  -b nm             Split a file into n*1048576 bytes in size\n"
   "  -l line_count     Number of lines in each file\n"
   "  -line_count       Number of lines in each file\n"
   "  --help            Print this help message\n"
   "  --version         Print the program version\n\n"
   "Report bugs to fredlie@sprynet.com"
};

static char buf[BUF_SIZE];
static char *inname = 0;
static char *prog_name = 0;
static FILE *in = stdin;
static FILE *out = 0;
static char flag_lines = 0;
static int suffix_length = 2;

static FILE *file_open(char *name, int num)
{
   char sname[1024];
   if(num > 676)
   {
      fprintf(stderr, "%s: Exceeded maximum number of split files\n",
         prog_name);
      exit(1);
   }
   else
      sprintf(sname, "%s%c%c", name, num / 26 + 'a', num % 26 + 'a');
   return (fopen(sname, "wb"));
}

/*
 * Main
 */

int main(int argc, char **argv)
{
   int i;
   unsigned long size = 0;
   unsigned long num_lines = 0;
   char *ptr;

   prog_name = argv[0];
   
   for(i = 1; i < argc; ++i)
   {
      ptr = argv[i];
      if(*ptr == '-')
      {
         while(*++ptr)
         switch(*ptr)
         {
            case '-':
               ++ptr;
               if(!strcmp(ptr, "help"))
               {
                  puts(help_text);
                  return (0);
               }
               else if(!strcmp(ptr, "version"))
               {
                  puts("split (butils) "VERSION_STRING);
                  return (0);
               }
            case 'a':
               if(++i < argc)
                  sscanf(argv[i], "%d", &suffix_length);
               else
               {
                  fprintf(stderr, "%s: Missing argument in `-a' option.\n", 
                     argv[0]);
                  fputs("Try `split --help' for more information.\n", stderr);               
                  return (1);
               }
               break;
            case 'b':
               if(++i < argc)
               {
                  sscanf(argv[i], "%lu", &size);
                  ptr = argv[i];
                  while(*ptr != '\0')
                     ++ptr;
                  if(*--ptr == 'k')
                     size *= 1024;
                  else if(*ptr == 'm')
                     size *= 1048576;
               }
               else
               {
                  fprintf(stderr, "%s: Missing argument in `-b' option.\n", 
                     argv[0]);
                  fputs("Try `split --help' for more information.\n", stderr);               
                  return (1);
               }
               break;
            case 'l':
               if(++i < argc)
                  sscanf(argv[i], "%lu", &num_lines);
               else
               {
                  fprintf(stderr, "%s: Missing argument in `-l' option.\n", 
                     argv[0]);
                  fputs("Try `split --help' for more information.\n", stderr);               
                  return (1);
               }
               break;
            default:
               if((in = fopen(argv[i], "r")) == NULL)
               {
                  fprintf(stderr, "%s: Could not open `%s' for reading.\n", 
                     argv[0], argv[i]);
                  fputs("Try `split --help' for more information.\n", stderr);
                  return (1);
               }
               inname = argv[i];
         }
      }
      else
      {
         if((in = fopen(argv[i], "r")) == NULL)
         {
            fprintf(stderr, "%s: Could not open `%s' for reading.\n", 
               argv[0], argv[i]);
            fputs("Try `split --help' for more information.\n", stderr);
            return (1);
         }
         inname = argv[i];
      }
   }
   if(!flag_lines)
   {
      int number = 0;
      unsigned long bytes_left = size;
      unsigned long bytes_read = 0;      
      out = file_open(inname, number);
      
      while(1)
      {
         if(bytes_left >= BUF_SIZE)
            bytes_read = fread(buf, 1, BUF_SIZE, in);
         else
            bytes_read = fread(buf, 1, bytes_left, in);

         if(bytes_read <= 0)
         {
            fclose(out);
            fclose(in);
            return (0);
         }
         fwrite(buf, 1, bytes_read, out);
         bytes_left -= bytes_read;
         if(bytes_left == 0)
         {
            fclose(out);
            number++;
            out = file_open(inname, number);
            bytes_left = size;
         }
      }
   }
   return (0);
}